package storage.reporting;

import java.util.HashSet;
import java.util.Set;

/**
 * A concrete category which contains no subcategories.
 * 
 * @author mzi13
 * 
 */
public class ActivityCategory {
	// fields
	private String name;
	/**
	 * The name of the category. Should be user-friendly, as it may be displayed
	 * on a graph.
	 */
	private Set<ActivityCategory> subCategories;

	/** The subcategories of this category. */

	/**
	 * Constructor.
	 * 
	 * @param name
	 *            The name of this category.
	 */
	private ActivityCategory(String name) {
		this.name = name;
		subCategories = new HashSet<ActivityCategory>();
	}

	/**
	 * Constructor.
	 * 
	 * @param name
	 *            The name of this category.
	 */
	private ActivityCategory(String name, ActivityCategory[] subCategoryArray) {
		this.name = name;
		this.subCategories = new HashSet<ActivityCategory>();
		
		// add all needed subcategories.
		for(ActivityCategory s : subCategoryArray)
		{
			addSubCategory(s);
		}
	}
	
	/**
	 * Add a subcategory.
	 * 
	 * @param c
	 *            The subcategory to add.
	 */
	private void addSubCategory(ActivityCategory c) {
		subCategories.add(c);
	}

	/**
	 * Return all the subcategories of this category.
	 * 
	 * @return all the subcategories of this category.
	 */
	public Set<ActivityCategory> getSubCategories() {
		return subCategories;
	}
	
	/**
	 * Convert to a string.
	 * @return String representation.
	 */
	public String toString() {
		return name;
	}

	// define the single categories
	public static final ActivityCategory WRITING_CODE = new ActivityCategory(
			"Writing code");
	public static final ActivityCategory DESIGNING_CODE = new ActivityCategory(
			"Designing code");
	public static final ActivityCategory WRITING_TESTS = new ActivityCategory(
			"Writing tests");
	public static final ActivityCategory RUNNING_TESTS = new ActivityCategory(
			"Running tests");
	
	// define the composite categories
	public static ActivityCategory TESTING = new ActivityCategory(
		"Testing", new ActivityCategory[] {RUNNING_TESTS, WRITING_TESTS});
	public static ActivityCategory DEVELOPMENT = new ActivityCategory(
			"Testing", new ActivityCategory[] {WRITING_CODE, DESIGNING_CODE});

	/**
	 * Testing method.
	 */
	public static void main(String[] args)
	{
		for (ActivityCategory c : TESTING.getSubCategories())
		{
			System.out.println(c);
		}
	}
}
