package storage.reporting;

import java.awt.FlowLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.text.DateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sourceforge.jdatepicker.impl.UtilDateModel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.category.DefaultCategoryDataset;

/**
 * A new class for the COSC 427 Project. Represents the GUI side of the Report.
 * @author  mzi13
 */
public class ReportGUI implements Observer, ActionListener 
{
	/**
	 * @uml.property  name="report"
	 * @uml.associationEnd  
	 */
	private Report report; /** The managed report. */
	private org.jfree.chart.JFreeChart mChart;	/** The chart which displays the data. */
	
	/**
	 * Constructor. Add the report which needs to be observed.
	 */
	public ReportGUI()
	{

	}
	
	/**
	 * Create and return a wizard suitable for showing to the user.
	 * @return A wizard object that can be run in order to produce a nice chart appropriate for the type of report.
	 */
	public IWizard createWizard()
	{
		return new NetBeansWizard();
	}
	
	/** Update the GUI settings to reflect the status of the report. */
	@Override
	public void update(Observable o, Object arg) 
	{
		System.out.println("update() called for ReportGUI..");
		if(mChart != null)
		{
			mChart.setTitle(report.makeTitle());
		}
		
	}

	/**
	 * Set the report which is displayed by this GUI.
	 * @param  r
	 * @uml.property  name="report"
	 */
	public void setReport(Report r)
	{
		report = r;
		r.addObserver(this);
	}
	
	/**
	 * Show a chart which give the calculated data for the report.
	 * Modified by Martin, 19 Sep 2010. Changed to suit the DateModel I am using instead of a raw date.
	 */
    public void showChart()
    { 
        // create window title
        String windowTitle = "Patchwork Metrics report";
        
        // create axis labels
        String xAxisLabel = "Part";
        String yAxisLabel = "Time";
        
        // generate the chart data from the report
        // Modified by Martin, 19 Sep 2010. Added cast to date, since now I am using a DateModel object.
        ChartData data = report.getChartData();

        // iterate over the data and add it to the chart data
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        for(Map.Entry<String, Integer> entry : data.getMap().entrySet())
        {
            dataset.addValue(entry.getValue(), "Time Spent", entry.getKey());
        }

        // finally, create the chart and show it
        mChart = ChartFactory.createBarChart(report.makeTitle(), xAxisLabel, yAxisLabel, dataset, PlotOrientation.VERTICAL, false, true, false);
        //ChartFrame frame = new ChartFrame(windowTitle, mChart);
        ChartPanel chartPanel = new ChartPanel(mChart);

        // create a window
        JFrame frame = new JFrame();
        frame.add(chartPanel);
        frame.setContentPane(chartPanel);
        
        // add components to allow updates
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("Change Report Parameters");
        menu.setMnemonic(KeyEvent.VK_C);
        menu.getAccessibleContext().setAccessibleDescription(
                "The only menu in this program that has menu items");
        menuBar.add(menu);
        JMenuItem menuItem = new JMenuItem("Set start date to 1 January 2010");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_2, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("update");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        frame.setJMenuBar(menuBar);
        
        // lay out nicely and show
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * Show a wizard for the user to define the report settings with.
     * Also update the settings of the contained report after it is done.
     */
	public void showWizard() {
		IWizard w = createWizard();
		w.show();
		report.setStartDate(w.getStartDate());
		report.setEndDate(w.getEndDate());
	}

	/**
	 * Let the GUI update the report.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		System.out.println("Called action performed.");
		if(e.getActionCommand().equals("update"))
		{
			report.setStartDate(new Date(2010 - 1900, 0, 1));
		}
	}
}
